import _extends from "@babel/runtime/helpers/esm/extends";
var DomainBuilder = /*#__PURE__*/function () {
  DomainBuilder.extend = function extend(_temp) {
    var _ref = _temp === void 0 ? {
        domain: {}
      } : _temp,
      domain = _ref.domain,
      config = _ref.config;
    return new DomainBuilder({
      domain: domain,
      config: config
    });
  };
  function DomainBuilder(_temp2) {
    var _ref2 = _temp2 === void 0 ? {} : _temp2,
      domain = _ref2.domain,
      config = _ref2.config;
    this._domain = domain;
    this._useCase = false;
    this._config = config || domain.get('config');
    this._useCases = {};
  }
  var _proto = DomainBuilder.prototype;
  _proto.for = function _for(_temp3) {
    var _ref3 = _temp3 === void 0 ? {} : _temp3,
      useCase = _ref3.useCase;
    if (this._useCase) {
      throw new Error("[DomainBuilder#for] There is already an use case " + this._useCases + ".\n         Set up a response with DomainBuilder#respondWith before setting up another use case");
    }
    this._useCase = useCase;
    return this;
  };
  _proto.respondWith = function respondWith(_temp4) {
    var _ref4 = _temp4 === void 0 ? {} : _temp4,
      success = _ref4.success,
      fail = _ref4.fail;
    if (success !== undefined && fail !== undefined) {
      throw new Error('[DomainBuilder#respondWith] The response must set an object with success or fail prop, but not both');
    }
    if (success === undefined && fail === undefined) {
      throw new Error('[DomainBuilder#respondWith] Neither success nor fail are set');
    }
    if (!this._useCase) {
      throw new Error('[DomainBuilder#respondWith] before setting a response you must set up a use case using the DomainBuilder#for function');
    }
    this._useCases[this._useCase] = {
      success: success,
      fail: fail
    };
    this._useCase = false;
    return this;
  };
  _proto.build = function build(_temp5) {
    var _this = this;
    var _ref5 = _temp5 === void 0 ? {} : _temp5,
      inlineError = _ref5.inlineError;
    var self = this;
    var exeUseCase = function exeUseCase(useCase) {
      return function (params) {
        var subscribers = self._domain.subscribers || {};
        var subscriptions = subscribers[useCase] || [];
        var _self$_useCases$useCa = self._useCases[useCase],
          fail = _self$_useCases$useCa.fail,
          success = _self$_useCases$useCa.success;
        var data = typeof success === 'function' ? success(params) : success;
        var responseParams = data !== undefined ? {
          err: null,
          data: data
        } : {
          err: fail,
          data: null
        };
        var createResponse = function createResponse(_ref6) {
          var err = _ref6.err,
            data = _ref6.data;
          subscriptions.forEach(function (fn) {
            return fn({
              error: err,
              params: params,
              result: [err, data]
            });
          });
          if (inlineError) return Promise.resolve([err, data]);
          return err ? Promise.reject(err) : Promise.resolve(data);
        };
        return createResponse(responseParams);
      };
    };
    var buildGet = function buildGet(useCase) {
      if (useCase === 'config') {
        return _this._config;
      }
      var base = self._domain.get(useCase);
      if (self._useCases[useCase]) {
        return _extends({}, base, {
          execute: exeUseCase(useCase)
        });
      }
      return base;
    };
    return {
      get: buildGet,
      _map: Object.assign(self._domain._map || {}, self._useCases),
      useCases: Object.assign(self._domain.useCases || {}, self._useCases)
    };
  };
  return DomainBuilder;
}();
export { DomainBuilder as default };